#include "SamplingPreprocessor.h"
#include "ExactPreprocessor.h"
#include "Parser.h"
#include "UnigraphicsParser.h"
#include "X3dParser.h"
#include "Environment.h"
#include "Camera.h"
#include "MeshKdTree.h"
#include "Exporter.h"

#define USE_EXE_PATH false


int
main(int argc, const char **argv)
{
  Debug.open("debug.log");
  environment = new Environment;
  environment->Parse(argc, argv, USE_EXE_PATH);
  MeshKdTree::ParseEnvironment();
  BspTree::ParseEnvironment();

  Preprocessor *p =
    new SamplingPreprocessor();

  char buff[128];
  environment->GetStringValue("Scene.filename", buff);
  string filename(buff);

  p->LoadScene(filename);

  p->BuildKdTree();
  p->KdTreeStatistics(cout);

#ifdef TEST_BSP_VIEWCELLS
  environment->GetStringValue("Scene.viewcells", buff);

  string vcFilename(buff);

  // if BSP tree construction method needs predefined view cells
  if (BspTree::sConstructionMethod == BspTree::VIEW_CELLS)
  {
	  if (vcFilename != "")
	  {
          p->LoadViewCells(vcFilename);
		  
		  Exporter *exporter = Exporter::GetExporter("viewcells.x3d");
		  if (exporter) 
		  {	  exporter->ExportViewCells(&p->mViewCells);
			  delete exporter;  
		  }
	  }
	  else
		  p->GenerateViewCells();

	  Debug << "Number of view cells: " << p->mViewCells.size() << endl;
  }

  p->BuildBspTree();
  p->BspTreeStatistics(Debug);
  p->Export(filename + "-bsptree.x3d", false, false, true);

#endif

  //  p->mSceneGraph->Export("soda.x3d");
  if (0) {
    p->Export(filename + "-out.x3d", true, false, false);
    p->Export(filename + "-kdtree.x3d", false, true, false);	
  }
  
   
  if (1) {
    p->ComputeVisibility();
    p->ExportPreprocessedData("scene.vis");
  }

  if (1) {
    Camera camera;
    //camera.LookAtBox(p->mKdTree->GetBox());
	camera.LookInBox(p->mKdTree->GetBox());
	camera.SetPosition(camera.mPosition + Vector3(0,300,0));
    camera.SnapImage("camera.jpg", p->mKdTree);

    
    camera.LookInBox(p->mKdTree->GetBox());
    camera.SetPosition(camera.mPosition - Vector3(0,100,0));
    camera.SnapImage("camera2.png", p->mKdTree);
  }

  // clean up
  DEL_PTR(p);
  DEL_PTR(environment);

  return 0;
}

